#!/bin/env python3

# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import json
import os

from pathlib import Path
from typing import Dict, List, Any

IMAGESTREAMS_DIR: str = "imagestreams"


class ImageStreamChecker(object):
    version: str = ""

    def __init__(self, working_dir: Path = Path(".")):
        self.results: Dict[Any, Any] = {}
        self.working_dir = working_dir

    def get_latest_version(self) -> str:
        latest_version = ""
        print(f"Working dir for latest imagestream is {self.working_dir}.")
        with open(self.working_dir / "Makefile") as f:
            for line in f:
                if not line.startswith("VERSIONS ="):
                    continue
                versions = line.split("=")[1].strip().split()
                latest_version = versions[-1]
        print(f"The latest version is {latest_version}.")
        return latest_version

    def load_json_file(self, filename: Path) -> Any:
        with open(str(filename)) as f:
            data = json.load(f)
            isinstance(data, Dict)
            return data

    def check_version(self, json_dict: Dict[Any, Any]) -> List[str]:
        res = []
        for tags in json_dict["spec"]["tags"]:
            print(
                f"check_version: Compare tags['name']:'{tags['name']}' against version:'{self.version}'"
            )
            # The name can be"<stream>" or "<stream>-elX" or "<stream>-ubiX"
            if tags["name"] == self.version or tags["name"].startswith(
                self.version + "-"
            ):
                res.append(tags)
        return res

    def check_latest_tag(self, json_dict: Dict[Any, Any]) -> bool:
        latest_tag_correct: bool = False
        for tags in json_dict["spec"]["tags"]:
            if tags["name"] != "latest":
                continue
            print(
                f"check_latest_tag: Compare tags['name']:'{tags['name']}' against version:'{self.version}'"
            )
            # The latest can link to either "<stream>" or "<stream>-elX" or "<stream>-ubiX"
            if tags["from"]["name"] == self.version or tags["from"]["name"].startswith(
                self.version + "-"
            ):
                latest_tag_correct = True
                print("Latest tag found.")
        return latest_tag_correct

    def check_imagestreams(self, version: str) -> int:
        self.version = version
        p = Path(".")
        json_files = p.glob(f"{IMAGESTREAMS_DIR}/*.json")
        if not json_files:
            print(f"No json files present in {IMAGESTREAMS_DIR}.")
            return 0
        for f in json_files:
            if os.environ.get("TARGET") in ("rhel7", "centos7") and "aarch64" in str(f):
                print("Imagestream aarch64 is not supported on rhel7")
                continue
            print(f"Checking file {str(f)}.")
            json_dict = self.load_json_file(f)
            if not (self.check_version(json_dict) and self.check_latest_tag(json_dict)):
                print(
                    f"The latest version is not present in {str(f)} or in latest tag."
                )
                self.results[f] = False
        if self.results:
            return 1
        print("Imagestreams contains the latest version.")
        return 0



# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import json
import logging
import time

from subprocess import CalledProcessError
from typing import Dict, Any

from container_ci_suite.utils import run_oc_command


logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.DEBUG)
logger = logging.getLogger(__name__)


class OpenShiftOperations:
    namespace: str = ""
    pod_json_data: Dict = {}

    def __init__(self, pod_name_prefix: str = ""):
        self.pod_name_prefix = pod_name_prefix
        self.build_failed: bool = False

    def set_namespace(self, namespace: str):
        self.namespace = namespace

    def get_pod_status(self) -> Dict:
        # output = OpenShiftAPI.run_oc_command("get all", json_output=False)
        # print(f"oc get all: {output}")
        output = run_oc_command("get pods", json_output=True, namespace=self.namespace)
        # print(f" oc get pods: {output}")
        return json.loads(output)

    def print_get_status(self):
        print("Print get all and status:")
        print(run_oc_command("get all", namespace=self.namespace, json_output=False))
        print(run_oc_command("status", namespace=self.namespace, json_output=False))
        print(run_oc_command("status --suggest", namespace=self.namespace, json_output=False))

    def print_pod_logs(self):
        self.pod_json_data = self.get_pod_status()
        print("Print all pod logs")
        for item in self.pod_json_data["items"]:
            pod_name = item["metadata"]["name"]
            print(f"Logs from pod name {pod_name}:")
            oc_logs = run_oc_command(f"logs pod/{pod_name}", json_output=False)
            print(oc_logs)

    def is_project_exits(self) -> bool:
        output = run_oc_command("projects", json_output=False)
        if self.namespace in output:
            return True
        return False

    def get_pod_count(self) -> int:
        count: int = 0
        for item in self.pod_json_data["items"]:
            pod_name = item["metadata"]["name"]
            if self.pod_name_prefix not in pod_name:
                continue
            if "deploy" in pod_name:
                continue
            if "build" in pod_name:
                continue
            count += 1
        return count

    def get_logs(self, pod_name) -> str:
        return run_oc_command(
            f"logs {pod_name}", namespace=self.namespace, json_output=False
        )

    def is_pod_running(self, pod_name_prefix: str = "", loops: int = 180) -> bool:
        print(f"Check for POD is running {pod_name_prefix}")
        for count in range(loops):
            print(".", sep="", end="")
            self.pod_json_data = self.get_pod_status()
            if pod_name_prefix == "" and self.pod_name_prefix == "":
                print("\nApplication pod name is not specified. Call: is_pod_running(pod_name_prefix=\"something\").")
                return False
            if pod_name_prefix != "":
                self.pod_name_prefix = pod_name_prefix
            # Only one running pod is allowed
            if self.get_pod_count() != 1:
                time.sleep(3)
                continue

            for item in self.pod_json_data["items"]:
                pod_name = item["metadata"]["name"]
                if self.pod_name_prefix not in pod_name:
                    continue
                status = item["status"]["phase"]
                if "deploy" in pod_name:
                    print(".", sep="", end="")
                    continue
                if item["status"]["phase"] == "Running":
                    print(f"\nPod with name {pod_name} is running {status}.")
                    output = self.get_logs(pod_name=pod_name)
                    print(output)
                    # Wait couple seconds for sure
                    time.sleep(3)
                    return True
            time.sleep(3)
        return False

    def is_build_pod_present(self) -> bool:
        for item in self.pod_json_data["items"]:
            pod_name = item["metadata"]["name"]
            if "build" in pod_name:
                return True
        return False

    def is_pod_finished(self, pod_suffix_name: str = "deploy") -> bool:
        if not self.pod_json_data:
            self.pod_json_data = self.get_pod_status()
        for item in self.pod_json_data["items"]:
            print(".", sep="", end="")
            pod_name = item["metadata"]["name"]
            if self.pod_name_prefix not in pod_name:
                print(".", sep="", end="")
                continue
            status = item["status"]["phase"]
            if pod_suffix_name in pod_name and status == "Failed":
                print(f"\nPod with {pod_suffix_name} finished with {status}. See logs.")
                self.build_failed = True
                self.print_pod_logs()
                self.print_get_status()
                return False
            if pod_suffix_name in pod_name and status != "Succeeded":
                print(".", sep="", end="")
                continue
            if pod_suffix_name in pod_name and status == "Succeeded":
                print(f"\nPod with suffix {pod_suffix_name} is finished")
                return True
        return False

    def is_build_pod_finished(self, cycle_count: int = 180) -> bool:
        """
        Function return information if build pod is finished.
        The function waits for 180*3 seconds
        """
        print("Check if build pod is finished")
        for count in range(cycle_count):
            print(".", sep="", end="")
            self.pod_json_data = self.get_pod_status()
            if len(self.pod_json_data["items"]) == 0:
                time.sleep(3)
                continue
            if not self.is_build_pod_present():
                print(".", sep="", end="")
                time.sleep(3)
                continue
            if not self.is_pod_finished(pod_suffix_name="build"):
                print(".", sep="", end="")
                if self.build_failed:
                    return False
                time.sleep(3)
                continue
            print("\nBuild pod is finished")
            return True
        return False

    def is_s2i_pod_running(self, pod_name_prefix: str = "", cycle_count: int = 180) -> bool:
        self.pod_name_prefix = pod_name_prefix
        build_pod_finished = False
        print("Check if S2I build pod is running")
        for count in range(cycle_count):
            print(".", sep="", end="")
            self.pod_json_data = self.get_pod_status()
            if len(self.pod_json_data["items"]) == 0:
                time.sleep(3)
                continue
            if not self.is_build_pod_present():
                time.sleep(3)
                continue
            if not self.is_pod_finished(pod_suffix_name="build"):
                time.sleep(3)
                continue
            build_pod_finished = True
            print(f"\nBuild pod with name {pod_name_prefix} is finished.")
            break
        if not build_pod_finished:
            print(f"\nBuild pod with name {pod_name_prefix} was not finished.")
            return False
        print("Check if S2I pod is running.")
        for count in range(cycle_count):
            print(".", sep="", end="")
            if not self.is_pod_running():
                time.sleep(3)
                continue
            print("\nPod is running")
            return True
        return False

    def oc_get_services(self, service_name):
        output = run_oc_command(f"get svc/{service_name}", json_output=True, namespace=self.namespace)
        json_output = json.loads(output)
        print(json_output)
        return json_output

    def get_service_ip(self, service_name) -> Any:
        json_output = self.oc_get_services(service_name=service_name)
        if "clusterIP" not in json_output["spec"]:
            return None
        if not json_output["spec"]["clusterIP"]:
            return None
        return json_output["spec"]["clusterIP"]

    def is_imagestream_exist(self, name: str):
        try:
            json_output = self.oc_get_is(name=name)
            if json_output["kind"] == "ImageStream" and json_output["metadata"]["name"] == name:
                return json_output
        except CalledProcessError:
            pass
        return None

    def get_routes(self):
        output = run_oc_command(
            "get route",
            namespace=self.namespace, return_output=True, ignore_error=True, shell=True
        )
        return json.loads(output)

    def oc_gel_all_is(self):
        output = run_oc_command("get is", namespace=self.namespace)
        return json.loads(output)

    def oc_get_is(self, name: str):
        output = run_oc_command(f"get is/{name}", namespace=self.namespace)
        return json.loads(output)

    def check_is_exists(self, is_name, version_to_check: str) -> bool:
        """
        Function checks if it exists in OpenShift 4 environment
        Exact version has to be the same
        :return:    True if tag was found
                    False if tag does not exist
        """
        json_output = self.oc_get_is(name=is_name)
        if "tags" not in json_output["spec"]:
            return False
        tag_found: bool = False
        for tag in json_output["spec"]["tags"]:
            if "name" not in tag:
                continue
            if version_to_check not in tag["name"]:
                continue
            tag_found = True
        return tag_found

    def is_pod_ready(self, cycle_count: int = 60) -> bool:
        """
        Function checks if pod with specific name is really ready
        """
        print("Check if pod is ready.")
        for count in range(cycle_count):
            print(".", end="")
            json_data = self.get_pod_status()
            if len(json_data["items"]) == 0:
                time.sleep(3)
                continue
            if not self.is_pod_finished(pod_suffix_name=self.pod_name_prefix):
                time.sleep(3)
                continue
            for item in json_data["items"]:
                pod_name = item["metadata"]["name"]
                status = item["status"]["phase"]
                if "deploy" in pod_name:
                    continue
                if item["status"]["phase"] == "Running":
                    print(f"\nPod with name {pod_name} is running {status}.")
                    output = run_oc_command(
                        f"logs {pod_name}", namespace=self.namespace, json_output=False
                    )
                    print(output)
                    # Wait couple seconds for sure
                    time.sleep(10)
                    return True
                time.sleep(3)
        return False



#!/bin/env python3

# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import yaml

from flexmock import flexmock

import pytest

from container_ci_suite.utils import (
    get_public_image_name,
    get_npm_variables,
    get_mount_ca_file,
    get_mount_options_from_s2i_args,
    get_env_commands_from_s2i_args,
)
from container_ci_suite import utils
from tests.conftest import create_ca_file, delete_ca_file


class TestContainerCISuiteUtils(object):
    @pytest.mark.parametrize(
        "os,base_image_name,version,expected_str",
        [
            ("rhel7", "nodejs", "12", "registry.redhat.io/rhscl/nodejs-12-rhel7"),
            ("rhel8", "nodejs", "14", "registry.redhat.io/rhel8/nodejs-14"),
            ("centos7", "nodejs", "10", "docker.io/centos/nodejs-10-centos7"),
        ],
    )
    def test_get_public_image_name(self, os, base_image_name, version, expected_str):
        name = get_public_image_name(
            os=os, base_image_name=base_image_name, version=version
        )
        assert name == expected_str

    def test_get_npm_variables_no_ca_file(self):
        assert get_npm_variables() == ""

    def test_get_mount_ca_file_no_ca_file(self):
        assert get_mount_ca_file() == ""

    def test_get_npm_variables(self):
        create_ca_file()
        assert get_npm_variables() == f"-e NPM_MIRROR=foobar {get_mount_ca_file()}"
        delete_ca_file()

    def test_get_mount_ca_file(self):
        create_ca_file()
        assert get_mount_ca_file() == f"{get_mount_ca_file()}"
        delete_ca_file()

    @pytest.mark.parametrize(
        "s2i_args,expected_output",
        [
            ("--pull-never", ""),
            (
                "--pull-never -v /some/foo/bar/file:/some/foo/bar/file:Z",
                "-v /some/foo/bar/file:/some/foo/bar/file:Z",
            ),
        ],
    )
    def test_mount_point(self, s2i_args, expected_output):
        create_ca_file()
        ret_value = get_mount_options_from_s2i_args(s2i_args=s2i_args)
        assert ret_value == expected_output
        delete_ca_file()

    @pytest.mark.parametrize(
        "s2i_args,expected_output",
        [
            ("--pull-never", []),
            ("--pull-never -e NODE=development", ["ENV NODE=development"]),
            (
                "-v mount_point:mount_point:Z -e FOO=bar --env TEST=deployment",
                ["ENV FOO=bar", "ENV TEST=deployment"],
            ),
            ("-v mount_point:mount_point:Z -e FOO=bar --env TEST", ["ENV FOO=bar"]),
        ],
    )
    def test_get_env_from_s2i_args(self, s2i_args, expected_output):
        assert get_env_commands_from_s2i_args(s2i_args=s2i_args) == expected_output

    @pytest.mark.parametrize(
        "image_name,version,os_name,expected_output",
        [
            ("rhel8/ubi8", "", "", False),
            ("", "2.4", "", False),
            ("", "", "rhel8", False),
            ("rhel8/httpd-24:1", "2.4", "rhel8", True),
        ],
    )
    def test_check_variables(self, image_name, version, os_name, expected_output):
        os.environ["IMAGE_NAME"] = image_name
        os.environ["VERSION"] = version
        os.environ["OS"] = os_name
        assert utils.check_variables() == expected_output

    @pytest.mark.parametrize(
        "image_name,version,expected_output",
        [
            ("rhel8/httpd-24:1", "2.4", "httpd-24:2.4"),
            ("/httpd-24:1", "2.4", "httpd-24:2.4"),
            ("", "2.4", None),
            ("rhel8/httpd", "2.4", "httpd:2.4")
        ],
    )
    def test_tagged_image(self, image_name, version, expected_output):
        assert utils.get_tagged_image(image_name=image_name, version=version) == expected_output

    @pytest.mark.parametrize(
        "image_name,expected_output",
        [
            ("rhel8/httpd-24:1", "httpd-24-testing"),
            ("/httpd-24:1", "httpd-24-testing"),
            ("", None),
            ("rhel8/httpd", "httpd-testing")
        ],
    )
    def test_get_service_image(self, image_name, expected_output):
        assert utils.get_service_image(image_name=image_name) == expected_output

    def test_tenantnamespace_yaml(self):
        expected_yaml = {
            "apiVersion": "tenant.paas.redhat.com/v1alpha1",
            "kind": "TenantNamespace",
            "metadata": {
                "name": "123456",
                "namespace": "core-services-ocp--config"
            },
            "spec": {
                "type": "runtime",
                "roles": [
                        "namespace-admin",
                        "tenant-egress-admin"
                ],
                "network": {
                    "security-zone": "internal"
                }

            }
        }
        tenant_yaml = utils.save_tenant_namespace_yaml(project_name="123456")
        with open(tenant_yaml) as fd:
            yaml_load = yaml.safe_load(fd.read())
        assert yaml_load
        assert yaml_load["metadata"]["name"] == "123456"
        assert yaml_load == expected_yaml

    def test_tenantnegress_yaml(self):
        expected_yaml = {
            "apiVersion": "tenant.paas.redhat.com/v1alpha1",
            "kind": "TenantEgress",
            "metadata": {
                "name": "default",
                "namespace": "core-services-ocp--123456"
            },
            "spec": {
                "egress": [
                    {
                        "to": {
                            "dnsName": "github.com"
                        },
                        "type": "Allow"
                    },
                    {
                        "to": {
                            "cidrSelector": "172.0.0.0/8"
                        },
                        "type": "Allow"
                    },
                    {
                        "to": {
                            "cidrSelector": "172.0.0.0/8"
                        },
                        "type": "Allow"
                    },
                    {
                        "to": {
                            "cidrSelector": "10.0.0.0/9"
                        },
                        "type": "Allow"
                    },
                    {
                        "to": {
                            "cidrSelector": "52.218.128.0/17"
                        },
                        "type": "Allow"
                    },
                    {
                        "to": {
                            "cidrSelector": "52.92.128.0/17"
                        },
                        "type": "Allow"
                    },
                    {
                        "to": {
                            "cidrSelector": "52.216.0.0/15"
                        },
                        "type": "Allow"
                    }
                ]
            }
        }
        tenant_yaml = utils.save_tenant_egress_yaml(project_name="123456")
        print(tenant_yaml)
        with open(tenant_yaml) as fd:
            yaml_load = yaml.safe_load(fd.read())
        assert yaml_load
        assert yaml_load["metadata"]["namespace"] == "core-services-ocp--123456"
        assert yaml_load == expected_yaml

    @pytest.mark.parametrize(
        "os_env,expected_output",
        [
            ("False", False),
            ("false", False),
            (None, False),
            ("True", True),
            ("true", True),
            ("", False),
            ("Somthing", False),
            ("Y", True),
            ("y", True),
            ("1", True),
            ("No", False),
            ("0", False),
            ("true\n\n", True),
        ],
    )
    def test_is_shared_cluster(self, os_env, expected_output):
        flexmock(utils).should_receive("load_shared_credentials").and_return(os_env)
        assert utils.is_share_cluster() == expected_output



# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import json
import logging
import os
import re
import time
import random
import subprocess

from pathlib import Path
from typing import Dict, Any, List

from container_ci_suite.container import DockerCLIWrapper
from container_ci_suite.openshift_ops import OpenShiftOperations
from container_ci_suite.utils import run_oc_command


import container_ci_suite.utils as utils

logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.DEBUG)
logger = logging.getLogger(__name__)


class OpenShiftAPI:

    def __init__(
            self, namespace: str = "default",
            pod_name_prefix: str = "", create_prj: bool = True,
            delete_prj: bool = True,
            version: str = ""
    ):
        self.create_prj = create_prj
        self.delete_prj = delete_prj
        self.shared_cluster = utils.is_share_cluster()
        self.pod_name_prefix = pod_name_prefix
        self.pod_json_data: Dict = {}
        self.version = version
        self.shared_random_name = ""
        self.config_tenant_name = "core-services-ocp--config"
        self.openshift_ops = OpenShiftOperations(pod_name_prefix=pod_name_prefix)
        if namespace == "default":
            self.create_project()
        else:
            self.namespace = namespace
            self.create_prj = False

    def create_project(self):
        print(f"Create project {self.create_prj} and {self.shared_cluster}")
        if self.create_prj:
            if self.shared_cluster:
                self.shared_random_name = f"sclorg-{random.randrange(10000, 100000)}"
                self.namespace = f"core-services-ocp--{self.shared_random_name}"
                self.openshift_ops.set_namespace(self.namespace)
                if not self.prepare_tenant_namespace():
                    return False
            else:
                self.namespace = f"sclorg-{random.randrange(10000, 100000)}"
                self.openshift_ops.set_namespace(self.namespace)
                run_oc_command(f"new-project {self.namespace}", json_output=False, return_output=True)
                print(f"Project with the name '{self.namespace}' were created.")
        else:
            run_oc_command(f"project {self.namespace}", json_output=False)
        return self.openshift_ops.is_project_exits()

    def create_tenant_namespace(self) -> bool:
        tenant_yaml_file = utils.save_tenant_namespace_yaml(project_name=self.shared_random_name)
        try:
            tentant_output = run_oc_command(cmd=f"create -f {tenant_yaml_file}", json_output=False, return_output=True)
            print(tentant_output)
        except subprocess.CalledProcessError:
            print(f"Create tenant namespace with the name '{self.shared_random_name}' was not successful.")
            return False
        return True

    def create_egress_rules(self) -> bool:
        tenant_egress_file = utils.save_tenant_egress_yaml(project_name=self.shared_random_name)
        try:
            tentant_output = run_oc_command(cmd=f"apply -f {tenant_egress_file}", json_output=False, return_output=True)
            print(tentant_output)
        except subprocess.CalledProcessError as cpe:
            print(f"Apply egress rules to tenant namespace '{self.shared_random_name}' was not successful. {cpe}")
            return False
        return True

    def prepare_tenant_namespace(self):
        print(f"Prepare Tenant Namespace with name: '{self.shared_random_name}'")
        json_flag = False
        self.login_to_shared_cluster()
        if not self.create_tenant_namespace():
            return False
        # Let's wait 3 seconds till project is not up
        time.sleep(3)
        self.create_egress_rules()
        run_oc_command(
            cmd=f"project {self.namespace}",
            json_output=json_flag,
            return_output=True
        )
        print("Tenant Namespace were created")

    def delete_tenant_namespace(self):
        json_flag = False
        namespace = run_oc_command(cmd="project -q", json_output=json_flag)
        if namespace == self.config_tenant_name:
            print(f"Deleting tenant '{self.config_tenant_name}' is not allowed.")
            return
        run_oc_command(f"project {self.config_tenant_name}", json_output=json_flag)
        if run_oc_command(
                f"delete tenantnamespace {self.shared_random_name}",
                json_output=json_flag
        ):
            print(f"TenantNamespace {self.shared_random_name} was deleted properly")
        else:
            print(f"!!!!! TenantNamespace ${self.shared_random_name} was not delete properly."
                  f"But it does not block CI.!!!!")

    def get_raw_url_for_json(self, container: str, dir: str, filename: str, branch: str = "master"):
        return utils.get_raw_url_for_json(container=container, dir=dir, filename=filename, branch=branch)

    def is_s2i_pod_running(self, pod_name_prefix: str):
        return self.openshift_ops.is_s2i_pod_running(pod_name_prefix=pod_name_prefix)

    def is_pod_running(self, pod_name_prefix: str):
        return self.openshift_ops.is_pod_running(pod_name_prefix=pod_name_prefix)

    def delete_project(self):
        if not self.delete_prj:
            print("Deleting project is SUPPRESSED.")
            # project is not deleted by request user
        else:
            if self.shared_cluster:
                print("Delete project on shared cluster")
                self.delete_tenant_namespace()
            else:
                print(f"Deleting project {self.namespace}")
                run_oc_command("project default", json_output=False)
                run_oc_command(
                    f"delete project {self.namespace} --grace-period=0 --force", json_output=False
                )

    def run_command_in_pod(self, pod_name, command: str = "") -> str:
        output = run_oc_command(f"exec {pod_name} -- \"{command}\"")
        print(output)
        return output

    def import_is(self, path: str, name: str, skip_check=False):
        if not skip_check:
            is_exists = self.openshift_ops.is_imagestream_exist(name=name)
            if is_exists:
                return is_exists
        output = run_oc_command(f"create -f {path}", namespace=self.namespace)
        # Let's wait 3 seconds till imagestreams are not uploaded
        time.sleep(3)
        return json.loads(output)

    def process_file(self, path: str):
        output = run_oc_command(f"process -f {path}", namespace=self.namespace)
        json_output = json.loads(output)
        print(json_output)
        return json_output

    def start_build(self, service_name: str, app_name: str = "") -> str:
        from_dir = utils.download_template(template_name=app_name)
        output = run_oc_command(f"start-build {service_name} --from-dir={from_dir}", json_output=False)
        return output

    def login_to_shared_cluster(self):
        token = utils.load_shared_credentials("SHARED_CLUSTER_TOKEN")
        url = utils.load_shared_credentials("SHARED_CLUSTER_URL")
        if not all([token, url]):
            print("Important variables 'SHARED_CLUSTER_TOKEN,SHARED_CLUSTER_URL' are missing.")
            return None
        output = run_oc_command(f"login --token={token} --server={url}", json_output=False)
        print(output)
        output = run_oc_command("version", json_output=False)
        print(output)
        output = run_oc_command(f"project {self.config_tenant_name}", json_output=False)
        print(output)

    @staticmethod
    def login_external_registry() -> Any:
        registry_url = os.getenv("INTERNAL_IMAGE_REGISTRY", None)
        robot_token = utils.load_shared_credentials("ROBOT_TOKEN")
        robot_name = utils.load_shared_credentials("ROBOT_NAME")
        if not all([registry_url, robot_token, robot_name]):
            print("Important variables 'INTERNAL_IMAGE_REGISTRY,ROBOT_TOKEN,ROBOT_NAME' are missing.")
            return None
        cmd = f"podman login -u {robot_name} -p {robot_token} {registry_url}"
        output = utils.run_command(
            cmd=cmd,
            ignore_error=False,
            return_output=True
        )
        print(f"Output from podman login: {output}")
        return registry_url

    def upload_image_to_external_registry(self, source_image: str, tagged_image: str):
        register_url = OpenShiftAPI.login_external_registry()
        print(f"Registry_url: {register_url}")
        if not register_url:
            return None
        output_name = f"{register_url}/core-services-ocp/{tagged_image}"
        print(utils.run_command("podman images"))
        cmd = f"podman tag {source_image} {output_name}"
        output = utils.run_command(cmd, ignore_error=False, return_output=True)
        print(f"Output from podman tag command {output}")
        cmd = f"podman push {output_name}"
        output = utils.run_command(cmd, ignore_error=False, return_output=True)
        print(f"Output from podman push command {output}")
        ret = run_oc_command(
            f"import-image {tagged_image} --from={output_name} --confirm", json_output=False, return_output=True
        )
        print(ret)
        # Let's wait couple seconds
        time.sleep(3)
        return True

    def docker_login_to_openshift(self) -> Any:
        output = run_oc_command(cmd="get route default-route -n openshift-image-registry")
        jsou_output = json.loads(output)
        print(jsou_output["spec"]["host"])
        if not jsou_output["spec"]["host"]:
            print("Default route does not exist. Install OpenShift 4 cluster properly and expose default route.")
            return None
        ocp4_register = jsou_output["spec"]["host"]
        token_output = run_oc_command(cmd="whoami -t", json_output=False).strip()
        cmd = f"docker login -u kubeadmin -p {token_output} {ocp4_register}"
        output = utils.run_command(
            cmd=cmd,
            ignore_error=False,
            return_output=True
        )
        print(f"Output from docker login: {output}")
        return ocp4_register

    def upload_image(self, source_image: str, tagged_image: str) -> bool:
        """
        Function pull the image specified by parameter source_image
        and tagged it into OpenShift environment as tagged_image
        :param source_image: image that is pulled and uploaded to OpenShift
        :param tagged_image: image uploaded to OpenShift and tagged by this parameter
        :return True: image was properly uploaded to OpenShift
                False: image was not either pulled or uploading failed
        """
        if not DockerCLIWrapper.docker_pull_image(image_name=source_image, loops=3):
            return False
        try:
            ocp4_register = self.docker_login_to_openshift()
            if not ocp4_register:
                return False
        except subprocess.CalledProcessError:
            return False
        output_name = f"{ocp4_register}/{self.namespace}/{tagged_image}"
        cmd = f"docker tag {source_image} {output_name}"
        print(f"Tag docker image {cmd}")
        output = utils.run_command(
            cmd=cmd,
            ignore_error=False
        )
        print(f"Upload_image tagged {output}")
        output = utils.run_command(
            cmd=f"docker push {output_name}",
            ignore_error=False
        )
        print(f"Upload_image push {output}")
        return True

    def create_new_app_with_template(self, name: str, template_json: str, template_args: Dict = None):
        """
        Function creates a new application in OpenShift 4 environment
        :param name: str - Template name
        :param template_json: str - Template path to file
        :param template_args: Dict - Arguments that will be passed to oc new-app
        :return json toutput
        """

        # Let's wait couple seconds till is fully loaded
        time.sleep(3)
        args = [""]
        if template_args:
            args = [f"-p {key}={val}" for key, val in template_args]
        output = run_oc_command(
            f"new-app {template_json} --name {name} -p NAMESPACE={self.namespace} {args}",
            json_output=False
        )
        print(output)
        return output

    def get_route_url(self, routes_name: str) -> Any:
        output = run_oc_command(f"get routes/{routes_name}")
        json_output = json.loads(output)
        if not json_output["spec"]["host"]:
            return None
        if routes_name != json_output["spec"]["to"]["name"]:
            return None
        return json_output["spec"]["host"]

    def get_openshift_args(self, oc_args: List[str]) -> str:
        return " -p ".join(oc_args)

    def template_deployed(self, name_in_template: str = "") -> bool:
        if not self.openshift_ops.is_build_pod_finished():
            print("\nBuild pod does not finished in proper time")
            self.openshift_ops.print_get_status()
            return False
        if not self.openshift_ops.is_pod_running(pod_name_prefix=name_in_template):
            print("Pod is not running after time.")
            self.openshift_ops.print_get_status()
            return False
        return True

    def command_app_run(self, cmd: str, return_output: bool = True) -> str:
        cmd = f"exec command-app -- bash -c \"{cmd}\""
        print(f"command_app_run: {cmd}")
        cmd_out = run_oc_command(
            cmd=cmd, ignore_error=True, return_output=return_output, json_output=False
        )
        return cmd_out

    def create_deploy_command_app(self, image_name: str = "registry.access.redhat.com/ubi8/ubi") -> bool:
        cmd_file = utils.save_command_yaml(image_name=image_name)
        run_oc_command(f"create -f {cmd_file}")
        if not self.openshift_ops.is_pod_running(pod_name_prefix="command-app"):
            print("create_deploy_command_app: command-app pod is not running after time.")
            self.openshift_ops.print_get_status()
            return False
        output_cmd = self.command_app_run("echo $((11*11))")
        if "121" not in output_cmd:
            return False
        return True

    def imagestream_quickstart(
            self, imagestream_file: str, template_file: str,
            image_name: str, name_in_template: str, openshift_args=None
    ) -> bool:
        local_imagestream_file = utils.download_template(imagestream_file)
        self.import_is(local_imagestream_file, name="", skip_check=True)
        tagged_image = utils.get_tagged_image(image_name=image_name, version=self.version)
        if self.shared_cluster:
            if not self.upload_image_to_external_registry(source_image=image_name, tagged_image=tagged_image):
                return False
        else:
            if not self.upload_image(source_image=image_name, tagged_image=tagged_image):
                return False
        return self.deploy_template_with_image(
            image_name=image_name, template=template_file, name_in_template=name_in_template,
            openshift_args=openshift_args
        )

    def deploy_s2i_app(self, image_name: str, app: str, context: str, service_name: str = "") -> bool:
        tagged_image = utils.get_tagged_image(image_name=image_name, version=self.version)
        print(f"Source image {image_name} was tagged as {tagged_image}")
        if self.shared_cluster:
            if not self.upload_image_to_external_registry(source_image=image_name, tagged_image=tagged_image):
                return False
        else:
            if not self.upload_image(source_image=image_name, tagged_image=tagged_image):
                return False

        if service_name == "":
            service_name = utils.get_service_image(image_name)
        print(f"Service name in app is: {service_name}")
        app_param = app
        if Path(app).is_dir():
            app_param = utils.download_template(template_name=app)
        oc_cmd = f"new-app {tagged_image}~{app_param} --strategy=source --context-dir={context} --name={service_name}"
        try:
            output = run_oc_command(f"{oc_cmd}", json_output=False)
            print(output)
        except subprocess.CalledProcessError as cpe:
            print(cpe.output)
            return False

        time.sleep(5)
        if Path(app).is_dir():
            output = self.start_build(service_name=service_name, app_name=app)
            print(f"Output from start build: {output}")

        return True

    def deploy_image_stream_template(
            self, imagestream_file: str, template_file: str, app_name: str, openshift_args=None
    ) -> bool:
        local_is_file = utils.download_template(template_name=imagestream_file)
        local_template = utils.download_template(template_name=template_file)
        json_output = self.import_is(local_is_file, name="", skip_check=True)
        if not json_output:
            print("deploy_image_stream_template: import_is failed")
            return False
        if openshift_args is None:
            openshift_args = ""
        else:
            openshift_args = f"-p {self.get_openshift_args(oc_args=openshift_args)}"
        print(f"========\n"
              f"Creating a new-app with name {app_name} in "
              f"namespace {self.namespace} with args {openshift_args}\n"
              f"========")
        oc_cmd = f"new-app -f {local_template} --name={app_name} -p NAMESPACE={self.namespace} {openshift_args}"
        print(f"Deploy template by command: oc {oc_cmd}")
        try:
            output = run_oc_command(f"{oc_cmd}", json_output=False)
            print(output)
        except subprocess.CalledProcessError:
            return False
        # Let's wait couple seconds to deployment can start
        time.sleep(3)
        return True

    def deploy_imagestream_s2i(
            self, imagestream_file: str, image_name: str, app: str, context: str
    ) -> bool:
        """
        Function deploys imagestreams as s2i application
        In case of failure check if imagestream_file really exist
        :param imagestream_file: imagestream file that is imported to OCP4
        :param image_name: image name that is used for testing
        :param app: the app reference that is used in template like https://github.com/sclorg/httpd-ex.git
        :param context: specify context of in source git repository
        :return True: application was properly deployed
                False: application was not properly deployed
        """
        imagestream_file = re.sub(r"[0-9]", "", imagestream_file)
        local_template = utils.download_template(template_name=imagestream_file)
        if not local_template:
            return False
        self.import_is(local_template, name="", skip_check=True)
        return self.deploy_s2i_app(image_name=image_name, app=app, context=context)

    def deploy_template_with_image(
            self, image_name: str, template: str, name_in_template: str = "", openshift_args=None
    ) -> bool:
        tagged_image = f"{name_in_template}:{self.version}"
        if self.shared_cluster:
            if not self.upload_image_to_external_registry(source_image=image_name, tagged_image=tagged_image):
                return False
        else:
            if not self.upload_image(source_image=image_name, tagged_image=tagged_image):
                return False

        return self.deploy_template(
            template=template, name_in_template=name_in_template, openshift_args=openshift_args, expected_output=""
        )

    def deploy_template(
            self, template: str,
            name_in_template: str,
            expected_output: str,
            port: int = 8080,
            protocol: str = "http",
            response_code: int = 200,
            openshift_args=None,
            other_images=None
    ) -> bool:

        if other_images is None:
            other_images = ""
        if openshift_args is None:
            openshift_args = ""
        else:
            openshift_args = f"-p {self.get_openshift_args(openshift_args)}"
        print(f"========\n"
              f"Creating a new-app with name {name_in_template} in "
              f"namespace {self.namespace} with args ${openshift_args}\n"
              f"========")
        local_template = utils.download_template(template_name=template)
        if not local_template:
            return False
        oc_cmd = f"new-app {local_template} --name {name_in_template} -p NAMESPACE={self.namespace} {openshift_args}"
        print(f"Deploy template by command: oc {oc_cmd}")
        try:
            output = run_oc_command(f"{oc_cmd}", json_output=False)
            print(output)
        except subprocess.CalledProcessError:
            return False
        # Let's wait couple seconds to deployment can start
        time.sleep(3)
        return True

    def check_command_internal(
            self,
            image_name: str,
            service_name: str,
            cmd: str,
            expected_output: str,
            timeout: int = 120
    ) -> bool:
        if not self.create_deploy_command_app(image_name=image_name):
            return False
        ip_address = self.openshift_ops.get_service_ip(service_name=service_name)
        cmd = cmd.replace("<IP>", ip_address)
        for count in range(timeout):
            output = self.command_app_run(cmd=cmd, return_output=True)
            if expected_output in output:
                return True
            print(f"Output {expected_output} in NOT present in the output of `{cmd}`")
            time.sleep(3)
        return False

    def check_response_inside_cluster(
            self, cmd_to_run: str = None, name_in_template: str = "",
            expected_output: str = "",
            port: int = 8080,
            protocol: str = "http",
            response_code: int = 200,
            max_tests: int = 20
    ) -> bool:
        ip_address = self.openshift_ops.get_service_ip(service_name=name_in_template)
        url = f"{protocol}://{ip_address}:{port}/"
        print(f"URL address to get internal response is: {url}")
        if not self.create_deploy_command_app():
            return False
        if cmd_to_run is None:
            cmd_to_run = "curl --connect-timeout 10 -k -s -w '%{http_code}' " + f"{url}"
        # Check if application returns proper HTTP_CODE
        print("Check if HTTP_CODE is valid.")
        for count in range(max_tests):
            output_code = self.command_app_run(cmd=f"{cmd_to_run}", return_output=True)
            return_code = output_code[-3:]
            try:
                int_ret_code = int(return_code)
                if int_ret_code == response_code:
                    print(f"HTTP_CODE is VALID {int_ret_code}")
                    break
            except ValueError:
                print(return_code)
                time.sleep(3)
                continue
            time.sleep(5)
            continue

        cmd_to_run = "curl --connect-timeout 10 -k -s " + f"{url}"
        # Check if application returns proper output
        for count in range(max_tests):
            output_code = self.command_app_run(cmd=f"{cmd_to_run}", return_output=True)
            print(f"Check if expected output {expected_output} is in {cmd_to_run}.")
            if expected_output in output_code:
                print(f"Expected output '{expected_output}' is present.")
                return True
            print(
                f"check_response_inside_cluster:"
                f"expected_output {expected_output} not found in output of {cmd_to_run} command. See {output_code}"
            )
            time.sleep(10)
        return False

    def check_response_outside_cluster(
            self, name_in_template: str = "",
            expected_output: str = "",
            port: int = None,
            protocol: str = "http",
            response_code: int = 200,
    ) -> bool:

        route_name = self.get_route_url(routes_name=name_in_template)
        print(f"Route name is {route_name}")
        url = f"{protocol}://{route_name}"
        for count in range(3):
            print(f"Let's try to get response from route {url} one more time {count}")
            response_status = utils.get_response_request(
                url_address=url, response_code=response_code, expected_str=expected_output
            )
            if not response_status:
                time.sleep(10)
                continue
            break

        # ct_os_service_image_info
        return response_status



# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import pytest
import os
from flexmock import flexmock
from pathlib import Path


from container_ci_suite.helm import HelmChartsAPI
from container_ci_suite.openshift import OpenShiftAPI
from container_ci_suite.openshift_ops import OpenShiftOperations

test_dir = Path(os.path.abspath(os.path.dirname(__file__)))


class TestContainerCISuiteHelmCharts:

    def setup_method(self):
        flexmock(OpenShiftAPI).should_receive("create_project").and_return(True)
        flexmock(HelmChartsAPI).should_receive("set_namespace").and_return("something")
        self.helm_chart = HelmChartsAPI(
            Path("foo_path"), package_name="postgresql-imagestreams", tarball_dir=test_dir
        )

        self.helm_chart.oc_api.namespace = "something"
        self.helm_chart.namespace = "something"

    def test_helm_api(self):
        assert self.helm_chart.path == Path("foo_path")
        assert self.helm_chart.package_name == "postgresql-imagestreams"

    @pytest.mark.parametrize(
        "tag,registry,expected_value",
        [
            ("13-el9", "registry.redhat.io/rhel9/postgresql-13:latest", True),
            ("13-el8", "registry.redhat.io/rhel8/postgresql-13:latest", True),
            ("13-el6", "registry.redhat.io/rhel8/postgresql-13:latest", False),
            ("13-el8", "registry.redhat.io/rhel7/postgresql-13:latest", False),
        ]
    )
    def test_check_imagestreams(self, tag, registry, expected_value, postgresql_json):
        flexmock(OpenShiftOperations).should_receive("oc_gel_all_is").and_return(postgresql_json)
        self.helm_chart.tag = tag
        assert self.helm_chart.check_imagestreams(tag, registry=registry) == expected_value

    def test_package_helm_chart_success(self, helm_package_success, get_chart_yaml):
        flexmock(HelmChartsAPI).should_receive("is_chart_yaml_present").and_return(True)
        flexmock(HelmChartsAPI).should_receive("run_helm_command").and_return(helm_package_success)
        flexmock(HelmChartsAPI).should_receive("get_version_from_chart_yaml").and_return("0.0.1")
        assert self.helm_chart.package_name == "postgresql-imagestreams"
        assert self.helm_chart.helm_package()

    def test_package_helm_chart_failed(self, helm_package_failed):
        flexmock(HelmChartsAPI).should_receive("run_helm_command").and_return(helm_package_failed)
        assert not self.helm_chart.helm_package()

    # @pytest.mark.parametrize(
    #     "list_output,expected_output",
    #     [
    #         (True, True),
    #         (False, False),
    #     ]
    # )
    # def test_package_installation_success(
    #         self, package_installation_json, helm_list_json, list_output, expected_output
    # ):
    #     flexmock(HelmChartsAPI).should_receive("is_helm_package_installed").and_return(False)
    #     flexmock(HelmChartsAPI).should_receive("get_helm_json_output").and_return(package_installation_json)
    #     flexmock(HelmChartsAPI).should_receive("check_helm_installation").and_return(list_output)
    #     assert self.helm_chart.helm_installation() == expected_output



#!/bin/env python3

# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from flexmock import flexmock

from container_ci_suite.openshift_ops import OpenShiftOperations


class TestOpenShiftOpsSuite(object):
    def setup_method(self):
        self.oc_ops = OpenShiftOperations()
        self.oc_ops.set_namespace(namespace="container-ci-suite-test")

    def test_check_is_version(self, oc_get_is_ruby_json):
        flexmock(OpenShiftOperations).should_receive("oc_get_is").and_return(oc_get_is_ruby_json)
        assert self.oc_ops.check_is_exists("ruby", "2.5-ubi8")
        assert not self.oc_ops.check_is_exists("ruby", "333-ubi9")

    # TODO variant with outputs
    def test_get_pod_count(self, oc_build_pod_finished_json):
        self.oc_ops.pod_json_data = oc_build_pod_finished_json
        self.oc_ops.pod_name_prefix = "python-311-testing"
        assert self.oc_ops.get_pod_count() == 1

    def test_is_pod_running(self, oc_is_pod_running):
        flexmock(OpenShiftOperations).should_receive("get_pod_status").and_return(oc_is_pod_running)
        flexmock(OpenShiftOperations).should_receive("get_pod_count").and_return(1)
        flexmock(OpenShiftOperations).should_receive("get_logs").and_return("something")
        assert self.oc_ops.is_pod_running(pod_name_prefix="python-311", loops=2)

    def test_build_pod_not_finished(self, oc_build_pod_not_finished_json):
        flexmock(OpenShiftOperations).should_receive("get_pod_status").and_return(oc_build_pod_not_finished_json)
        assert not self.oc_ops.is_build_pod_finished(cycle_count=2)

    def test_build_pod_finished(self, oc_build_pod_finished_json):
        flexmock(OpenShiftOperations).should_receive("get_pod_status").and_return(oc_build_pod_finished_json)
        assert self.oc_ops.is_build_pod_finished(cycle_count=2)

    def test_get_service_ip(self, get_svc_ip):
        flexmock(OpenShiftOperations).should_receive("oc_get_services").and_return(get_svc_ip)
        assert self.oc_ops.get_service_ip("python-testing") == "172.30.224.217"

    def test_get_service_ip_not_available(self, get_svc_ip_empty):
        flexmock(OpenShiftOperations).should_receive("oc_get_services").and_return(get_svc_ip_empty)
        assert self.oc_ops.get_service_ip("python-testing") is None

    def test_get_pod_status(self):
        # self.oc_api.get_pod_status()
        pass

    def test_is_s2i_pod_running(self):
        # self.oc_api.is_s2i_pod_running()
        pass



# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.
import time

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from container_ci_suite.utils import run_command


class DockerCLIWrapper(object):
    @staticmethod
    def run_docker_command(
        cmd, return_output: bool = True, ignore_error: bool = False, shell: bool = True
    ):
        """
        Run docker command:
        """
        return run_command(
            f"podman {cmd}",
            return_output=return_output,
            ignore_error=ignore_error,
            shell=shell,
        )

    @staticmethod
    def docker_image_exists(image_name: str) -> bool:
        """
        Check if docker image exists or not
        :param image_name: image to check
        :return True: In case if image is present
                False: In case if image is not present
        """
        output = DockerCLIWrapper.run_docker_command(
            f"images -q {image_name}", ignore_error=True, return_output=True)
        return True if output != "" else False

    @staticmethod
    def docker_inspect(field: str, src_image: str) -> str:
        return DockerCLIWrapper.run_docker_command(
            f"docker inspect -f '{field}' {src_image}"
        )

    @staticmethod
    def docker_run_command(cmd):
        return DockerCLIWrapper.run_docker_command(f"run {cmd}")

    @staticmethod
    def docker_get_user_id(src_image, user):
        return DockerCLIWrapper.docker_run_command(
            f"--rm {src_image} bash -c 'id -u {user} 2>/dev/null"
        )

    @staticmethod
    def docker_pull_image(image_name: str, loops: int = 10) -> bool:
        """
        Function checks if image_name is present in system.
        In case it isn't, try to pull it for specific count of loops
        Default is 10.
        """
        if DockerCLIWrapper.docker_image_exists(image_name=image_name):
            print("Pulled image already exists.")
            return True
        for loop in range(loops):
            ret_val = DockerCLIWrapper.run_docker_command(
                cmd=f"pull {image_name}", return_output=False
            )
            if ret_val == 0 and DockerCLIWrapper.docker_image_exists(image_name=image_name):
                return True
            DockerCLIWrapper.run_docker_command("images", return_output=True)
            print(f"Pulling of image {image_name} failed. Let's wait {loop*5} seconds and try again.")
            time.sleep(loop*5)
        return False



# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import logging
import shutil
import subprocess
import re
import time
import random
import string
import requests
import tempfile
import yaml

from typing import List, Any
from pathlib import Path

from container_ci_suite.constants import CA_FILE_PATH

logger = logging.getLogger(__name__)


def get_file_content(filename: Path) -> str:
    with open(str(filename)) as f:
        return f.read()


def get_full_ca_file_path() -> Path:
    return Path(CA_FILE_PATH)


def get_os_environment(variable: str) -> str:
    return os.getenv(variable)


def get_mount_ca_file() -> str:
    if get_os_environment("NPM_REGISTRY") and get_full_ca_file_path().exists():
        return f"-v {CA_FILE_PATH}:{CA_FILE_PATH}:Z"
    return ""


def get_npm_variables():
    npm_registry = get_os_environment("NPM_REGISTRY")
    if npm_registry and get_full_ca_file_path().exists():
        return f"-e NPM_MIRROR={npm_registry} {get_mount_ca_file()}"
    return ""


def get_registry_name(os_name: str) -> str:
    return "registry.redhat.io" if os_name.startswith("rhel") else "docker.io"


def get_mount_options_from_s2i_args(s2i_args: str) -> str:
    # Check if -v parameter is present in s2i_args and add it into docker build command
    searchObj = re.search(r"(-v \.*\S*)", s2i_args)
    logger.debug(searchObj)
    if not searchObj:
        return ""

    logger.debug(searchObj.group(1))
    mount_options = searchObj.group()

    logger.info(f"Mount options: {mount_options}")
    return searchObj.group()


def get_env_commands_from_s2i_args(s2i_args: str) -> List:
    matchObj = re.findall(r"(-e|--env)\s*(\S*)=(\S*)", s2i_args)
    logger.debug(matchObj)
    env_content: List = []
    if matchObj:
        env_content.extend([f"ENV {x[1]}={x[2]}" for x in matchObj])
        logger.debug(env_content)
        return env_content
    return env_content


def get_public_image_name(os: str, base_image_name: str, version: str) -> str:
    registry = get_registry_name(os)
    if os == "rhel7":
        return f"{registry}/rhscl/{base_image_name}-{version}-rhel7"
    elif os == "rhel8":
        return f"{registry}/rhel8/{base_image_name}-{version}"
    else:
        return f"{registry}/centos/{base_image_name}-{version}-centos7"


def download_template(template_name: str, dir_name: str = "/var/tmp") -> Any:
    ext = ""
    file_ext_field = template_name.split(".")
    if len(file_ext_field) > 1:
        ext = f".{file_ext_field[1]}"
    print(f"Local temporary file {template_name} with extension {ext}")
    print(f"Temporary file: download_template from {template_name}")
    random_text = ''.join(random.choices(string.ascii_uppercase + string.digits, k=10))
    path_name = f"{dir_name}/{random_text}{ext}"
    if Path(template_name).is_file():
        shutil.copy2(template_name, path_name)
        return str(path_name)
    if Path(template_name).is_dir():
        shutil.copytree(template_name, path_name, symlinks=True)
        return str(path_name)
    if template_name.startswith("http"):
        resp = requests.get(template_name, verify=False)
        resp.raise_for_status()
        if resp.status_code != 200:
            print(f"utils.download_template: {resp.status_code} and {resp.text}")
            return None
        with open(path_name, "wb") as fd:
            fd.write(resp.content)
        return str(path_name)
    if not Path(template_name).exists():
        print("File to download does not exist.")
        return None


def run_command(
    cmd,
    return_output: bool = True,
    ignore_error: bool = False,
    shell: bool = True,
    debug: bool = False,
    **kwargs,
):
    """
    Run provided command on host system using the same user as invoked this code.
    Raises subprocess.CalledProcessError if it fails.
    :param cmd: list or str
    :param return_output: bool, return output of the command
    :param ignore_error: bool, do not fail in case nonzero return code
    :param shell: bool, run command in shell
    :param debug: bool, print command in shell, default is suppressed
    :return: None or str
    """
    if debug:
        logger.debug(f"command: {cmd}")
    try:
        if return_output:
            return subprocess.check_output(
                cmd,
                stderr=subprocess.STDOUT,
                universal_newlines=True,
                shell=shell,
                **kwargs,
            )
        else:
            return subprocess.check_call(cmd, shell=shell, **kwargs)
    except subprocess.CalledProcessError as cpe:
        if ignore_error:
            if return_output:
                return cpe.output
            else:
                return cpe.returncode
        else:
            logger.error(f"failed with code {cpe.returncode} and output:\n{cpe.output}")
            raise cpe


def run_oc_command(
    cmd, json_output: bool = True, return_output: bool = True, ignore_error: bool = False, shell: bool = True,
        namespace: str = ""
):
    """
    Run docker command:
    """
    json_cmd = "-o json" if json_output else ""
    namespace_cmd = f"-n {namespace}" if namespace != "" else ""

    return run_command(
        f"oc {cmd} {namespace_cmd} {json_cmd}",
        return_output=return_output,
        ignore_error=ignore_error,
        shell=shell
    )


def get_response_request(url_address: str, expected_str: str, response_code: int = 200, max_tests: int = 3) -> bool:
    for count in range(max_tests):
        try:
            resp = requests.get(url_address, timeout=10, verify=False)
            resp.raise_for_status()
            print(f"Response code is {resp.status_code} and expected should be {response_code}")
            if resp.status_code == response_code and expected_str in resp.text:
                return True
            return False
        except requests.exceptions.HTTPError:
            print("get_response_request: Service is not yet available. Let's wait some time")
            pass
        except requests.exceptions.ConnectTimeout:
            print("get_response_request: ConnectTimeout. Let's wait some time")
            pass
        time.sleep(10)
    return False


def temporary_dir(prefix: str = "helm-chart") -> str:
    temp_file = tempfile.TemporaryDirectory(prefix=prefix)
    print(f"Temporary dir name: {temp_file.name}")
    return temp_file.name


def save_command_yaml(image_name: str) -> str:
    cmd_yaml = {
        "apiVersion": "v1",
        "kind": "Pod",
        "metadata": {
            "name": "command-app"
        },
        "spec": {
            "restartPolicy": "OnFailure",
            "containers": [
                {
                    "name": "command-container",
                    "image": image_name,
                    "command": ["sleep"],
                    "args": ["3h"]
                }
            ]
        }
    }
    temp_file = tempfile.NamedTemporaryFile(prefix="command-yml", delete=False)
    with open(temp_file.name, "w") as fp:
        yaml.dump(cmd_yaml, fp)
    print(f"Pod command yaml file: {temp_file.name}")
    return temp_file.name


def save_tenant_namespace_yaml(project_name: str) -> str:
    cmd_yaml = {
        "apiVersion": "tenant.paas.redhat.com/v1alpha1",
        "kind": "TenantNamespace",
        "metadata": {
            "name": f"{project_name}",
            "namespace": "core-services-ocp--config"
        },
        "spec": {
            "type": "runtime",
            "roles": [
                    "namespace-admin",
                    "tenant-egress-admin"
            ],
            "network": {
                "security-zone": "internal"
            }

        }
    }
    temp_file = tempfile.NamedTemporaryFile(prefix="tenant-namespace-yml", delete=False)
    with open(temp_file.name, "w") as fp:
        yaml.dump(cmd_yaml, fp)
    print(f"TenantNamespace yaml file: {temp_file.name}")
    return temp_file.name


def save_tenant_egress_yaml(project_name: str) -> str:
    tenant_egress_yaml = {
        "apiVersion": "tenant.paas.redhat.com/v1alpha1",
        "kind": "TenantEgress",
        "metadata": {
            "name": "default",
            "namespace": f"core-services-ocp--{project_name}"
        },
        "spec": {
            "egress": [
                {
                    "to": {
                        "dnsName": "github.com"
                    },
                    "type": "Allow"
                },
                {
                    "to": {
                        "cidrSelector": "172.0.0.0/8"
                    },
                    "type": "Allow"
                },
                {
                    "to": {
                        "cidrSelector": "172.0.0.0/8"
                    },
                    "type": "Allow"
                },
                {
                    "to": {
                        "cidrSelector": "10.0.0.0/9"
                    },
                    "type": "Allow"
                },
                {
                    "to": {
                        "cidrSelector": "52.218.128.0/17"
                    },
                    "type": "Allow"
                },
                {
                    "to": {
                        "cidrSelector": "52.92.128.0/17"
                    },
                    "type": "Allow"
                },
                {
                    "to": {
                        "cidrSelector": "52.216.0.0/15"
                    },
                    "type": "Allow"
                }
            ]
        }
    }
    temp_file = tempfile.NamedTemporaryFile(prefix="tenant-egress-yml", delete=False)
    with open(temp_file.name, "w") as fp:
        yaml.dump(tenant_egress_yaml, fp)
    print(f"TenantNamespace yaml file: {temp_file.name}")
    return temp_file.name


def get_tagged_image(image_name: str, version: str) -> Any:
    try:
        image_no_namespace = image_name.split('/')[1]
        image_no_tag = image_no_namespace.split(':')[0]
    except IndexError:
        return None

    return f"{image_no_tag}:{version}"


def get_service_image(image_name: str) -> Any:
    try:
        image_no_namespace = image_name.split('/')[1]
        image_no_tag = image_no_namespace.split(':')[0]
    except IndexError:
        return None
    return f"{image_no_tag}-testing"


def check_variables() -> bool:
    ret_value: bool = True
    if not os.getenv("IMAGE_NAME", None):
        print("Make sure IMAGE_NAME is defined")
        ret_value = False
    if not os.getenv("VERSION"):
        print("Make sure VERSION is defined")
        ret_value = False
    if not os.getenv("OS"):
        print("Make sure OS is defined")
        ret_value = False
    return ret_value


def load_shared_credentials(credential: str) -> Any:
    cread_path = os.environ.get(credential, None)
    if not cread_path:
        return None
    cred = ""
    with open(cread_path) as f:
        cred = f.read()
    if cred == "":
        return None
    return cred


def is_share_cluster() -> bool:
    file_shared_cluster = load_shared_credentials("SHARED_CLUSTER")
    if not file_shared_cluster:
        print("Not defined variable SHARED_CLUSTER")
        return False
    file_shared_cluster = ''.join(file_shared_cluster.split())
    if file_shared_cluster in ["True", "true", "1", "yes", "Yes", "y", "Y"]:
        print("Shared cluster allowed")
        return True
    print("\nShared cluster is not allowed.\nTo allow it add 'true' to file in variable $SHARED_CLUSTER.")
    return False


def get_raw_url_for_json(container: str, dir: str, filename: str, branch: str = "master") -> str:
    RAW_SCL_JSON_URL: str = "https://raw.githubusercontent.com/sclorg/{container}/{branch}/{dir}/{filename}"
    return RAW_SCL_JSON_URL.format(container=container, branch=branch, dir=dir, filename=filename)



import logging


class CCIFormatter(logging.Formatter):
    def format(self, record):
        if record.levelno == logging.INFO:
            self._style._fmt = (
                "%(asctime)s.%(msecs).03d %(filename)-17s %(levelname)-6s %(message)s"
            )
        elif record.levelno > logging.INFO:
            self._style._fmt = "%(levelname)-8s %(message)s"
        else:  # debug
            self._style._fmt = (
                "%(asctime)s.%(msecs).03d %(filename)-17s %(levelname)-6s %(message)s"
            )
        return logging.Formatter.format(self, record)


def set_logging(
    logger_name="cci",
    level=logging.INFO,
    handler_class=logging.StreamHandler,
    handler_kwargs=None,
    date_format="%H:%M:%S",
    work_dir="/var/tmp/container-ci-suite",
):
    """
    Set personal logger for this library.

    :param logger_name: str, name of the logger
    :param level: int, see logging.{DEBUG,INFO,ERROR,...}: level of logger and handler
    :param handler_class: logging.Handler instance, default is StreamHandler (/dev/stderr)
    :param handler_kwargs: dict, keyword arguments to handler's constructor
    :param date_format: str, date style in the logs
    :param work_dir: Path, directory with logs and results
    """
    if level != logging.NOTSET:
        logger = logging.getLogger(logger_name)
        logger.setLevel(level)
        logger.debug(f"Logging set to {logging.getLevelName(level)}")

        # do not read handlers if they are already present
        if not [x for x in logger.handlers if isinstance(x, handler_class)]:
            handler_kwargs = handler_kwargs or {}
            handler = handler_class(**handler_kwargs)
            handler.setLevel(level)

            formatter = CCIFormatter(None, date_format)
            handler.setFormatter(formatter)
            logger.addHandler(handler)

            if work_dir:
                # fh = logging.FileHandler(os.path.join(str(work_dir), 'nvr-checker.log'))
                fh = logging.FileHandler(f"{work_dir}/cci.log")
                fh.setLevel(logging.DEBUG)
                logger.addHandler(fh)
                fh.setFormatter(formatter)



#!/bin/env python3

# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import logging
import re
import time
import subprocess
import shutil

from typing import List
from os import getenv
from pathlib import Path
from tempfile import mkdtemp, mktemp

from container_ci_suite.container import DockerCLIWrapper
from container_ci_suite.helm import HelmChartsAPI
from container_ci_suite.utils import (
    run_command,
    get_file_content,
    get_mount_ca_file,
    get_full_ca_file_path,
    get_os_environment,
    get_mount_options_from_s2i_args,
    get_env_commands_from_s2i_args,
)

logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.DEBUG)
logger = logging.getLogger(__name__)


class ContainerCISuite(object):
    def __init__(self, image_name: str):
        self.image_name: str = image_name
        self.container_args: str = ""
        self.cid_file: Path = None
        self.cid_file_dir: Path = None
        logger.info(f"Image name to test: {image_name}")

    def s2i_usage(self):
        return DockerCLIWrapper.run_docker_command(
            f"run --rm {self.image_name} bash -c /usr/libexec/s2i/usage"
        )

    def is_image_available(self):
        return DockerCLIWrapper.run_docker_command(f"inspect {self.image_name}")

    def s2i_build_as_df(self, app_path: str, s2i_args: str, src_image, dst_image: str):
        named_tmp_dir = mkdtemp()
        tmp_dir = Path(named_tmp_dir)
        if tmp_dir.exists():
            logger.debug("Temporary Directory exists.")
        else:
            logger.debug("Temporary directory not exists.")
        ntf = mktemp(dir=str(tmp_dir), prefix="Dockerfile.")
        df_name = Path(ntf)
        df_content = self.s2i_create_df(
            tmp_dir=tmp_dir,
            app_path=app_path,
            s2i_args=s2i_args,
            src_image=src_image,
            dst_image=dst_image,
        )
        with open(df_name, mode="w") as f:
            f.writelines(df_content)
        mount_options = get_mount_options_from_s2i_args(s2i_args=s2i_args)
        # Run the build and tag the result
        DockerCLIWrapper.run_docker_command(
            f"build {mount_options} -f {df_name} --no-cache=true -t {dst_image}"
        )

    def s2i_create_df(
        self, tmp_dir: Path, app_path: str, s2i_args: str, src_image, dst_image: str
    ) -> List[str]:
        real_app_path = app_path.replace("file://", "")
        df_content: List = []
        local_scripts: str = "upload/scripts"
        local_app: str = "upload/src"
        os.chdir(tmp_dir)
        if not DockerCLIWrapper.docker_image_exists(src_image):
            if "pull-policy=never" not in s2i_args:
                DockerCLIWrapper.run_docker_command(f"pull {src_image}")

        user = DockerCLIWrapper.docker_inspect(
            field="{{.Config.User}}", src_image=src_image
        )
        if not user:
            user = "0"

        assert int(user)
        user_id = DockerCLIWrapper.docker_get_user_id(src_image=src_image, user=user)
        if not user_id:
            logger.error(f"id of user {user} not found inside image {src_image}.")
            logger.error("Terminating s2i build.")
            return None
        else:
            user_id = user

        incremental: bool = "--incremental" in s2i_args
        if incremental:
            inc_tmp = Path(mktemp(dir=str(tmp_dir), prefix="incremental."))
            run_command(f"setfacl -m 'u:{user_id}:rwx' {inc_tmp}")
            # Check if the image exists, build should fail (for testing use case) if it does not
            if not DockerCLIWrapper.docker_image_exists(src_image):
                return None
            # Run the original image with a mounted in volume and get the artifacts out of it
            cmd = (
                "if [ -s /usr/libexec/s2i/save-artifacts ];"
                ' then /usr/libexec/s2i/save-artifacts > "$inc_tmp/artifacts.tar";'
                ' else touch "$inc_tmp/artifacts.tar"; fi'
            )
            DockerCLIWrapper.run_docker_command(
                f"run --rm -v {inc_tmp}:{inc_tmp}:Z {dst_image} bash -c {cmd}"
            )
            # Move the created content into the $tmpdir for the build to pick it up
            shutil.move(f"{inc_tmp}/artifacts.tar", tmp_dir.name)
        real_local_app = tmp_dir / local_app
        real_local_scripts = tmp_dir / local_scripts
        os.makedirs(real_local_app.parent)
        shutil.copytree(real_app_path, real_local_app)
        bin_dir = real_local_app / ".s2i" / "bin"
        if bin_dir.exists():
            shutil.move(bin_dir, real_local_scripts)
        df_content.extend(
            [
                f"FROM {src_image}",
                f"LABEL io.openshift.s2i.build.image={src_image} "
                f"io.openshift.s2i.build.source-location={app_path}",
                "USER root",
                f"COPY {local_app}/ /tmp/src",
            ]
        )
        if real_local_scripts.exists():
            df_content.append(f"COPY {local_scripts} /tmp/scripts")
            df_content.append(f"RUN chown -R {user_id}:0 /tmp/scripts")
        df_content.append(f"RUN chown -R {user_id}:0 /tmp/src")

        # Check for custom environment variables inside .s2i/ folder
        env_file = Path(real_local_app / ".s2i" / "environment")
        if env_file.exists():
            with open(env_file) as fd:
                env_content = fd.readlines()
            # Remove any comments and add the contents as ENV commands to the Dockerfile
            env_content = [f"ENV {x}" for x in env_content if not x.startswith("#")]
            df_content.extend(env_content)

        # Filter out env var definitions from $s2i_args
        # and create Dockerfile ENV commands out of them
        env_content = get_env_commands_from_s2i_args(s2i_args=s2i_args)
        df_content.extend(env_content)

        # Check if CA autority is present on host and add it into Dockerfile
        if get_full_ca_file_path().exists():
            df_content.append(
                "RUN cd /etc/pki/ca-trust/source/anchors && update-ca-trust extract"
            )
        # Add in artifacts if doing an incremental build
        if incremental:
            df_content.extend(
                [
                    "RUN mkdir /tmp/artifacts",
                    "RUN artifacts.tar /tmp/artifacts",
                    f"RUN chown -R {user_id}:0 /tmp/artifacts",
                ]
            )
        df_content.append(f"USER {user_id}")
        # If exists, run the custom assemble script, else default to /usr/libexec/s2i/assemble
        if (real_local_scripts / "assemble").exists():
            df_content.append("RUN /tmp/scripts/assemble")
        else:
            df_content.append("RUN /usr/libexec/s2i/assemble")
        # If exists, set the custom run script as CMD, else default to /usr/libexec/s2i/run
        if Path(real_local_scripts / "run").exists():
            df_content.append("CMD /tmp/scripts/run")
        else:
            df_content.append("CMD /usr/libexec/s2i/run")

        logger.error(df_content)
        return df_content

    def scl_usage_old(self):
        pass

    def create_container(self, cid_file: str, container_args: str = "", *args):
        self.cid_file_dir = Path(mkdtemp(suffix=".test_cid_files"))
        p = Path(self.cid_file_dir)
        self.cid_file = p / cid_file
        DockerCLIWrapper.run_docker_command(
            f"run --cidfile={self.cid_file} -d {container_args} {self.image_name} {args}"
        )
        if not self.wait_for_cid():
            return False
        logger.info(f"Created container {self.get_cid_file()}")

    def wait_for_cid(self):
        max_attempts: int = 10
        attempt: int = 1
        while attempt < max_attempts:
            if self.cid_file.exists():
                with open(self.cid_file) as f:
                    logger.debug(f"{self.cid_file} contains:")
                    logger.info(f.readlines())
                return True
            logger.info("Waiting for container to start.")
            attempt += 1
            time.sleep(1)
        return False

    def get_cip(self):
        container_id = self.get_cid_file()
        return DockerCLIWrapper.run_docker_command(
            f"inspect --format='{{.NetworkSettings.IPAddress}}' {container_id}"
        )

    def check_envs_set(self):
        pass

    def get_cid_file(self, cid_file: Path = None):
        if cid_file is None:
            return get_file_content(self.cid_file)
        return get_file_content(cid_file)

    def check_image_availability(self, public_image_name: str):
        try:
            DockerCLIWrapper.run_docker_command(
                f"pull {public_image_name}", return_output=False
            )
        except subprocess.CalledProcessError as cfe:
            logger.error(f"{public_image_name} could not be downloaded via 'docker'.")
            logger.error(cfe)
            return False
        return True

    def cleanup_container(self):
        logger.info(f"Cleaning CID_FILE_DIR {self.cid_file_dir} is ongoing.")
        p = Path(self.cid_file_dir)
        cid_files = p.glob("*")
        for cid_file in cid_files:
            container_id = get_file_content(cid_file)
            logger.info("Stopping container")
            DockerCLIWrapper.run_docker_command(f"stop {container_id}")
            exit_code = DockerCLIWrapper.docker_inspect(
                field="{{.State.ExitCode}}", src_image=container_id
            )
            if exit_code != 0:
                logs = DockerCLIWrapper.run_docker_command(f"logs {container_id}")
                logger.info(logs)
            DockerCLIWrapper.run_docker_command(f"rm -v {container_id}")
            cid_file.unlink()
        os.rmdir(self.cid_file_dir)
        logger.info(f"Cleanning CID_FILE_DIR {self.cid_file_dir} is DONE.")

    def assert_container_fails(self, cid_file: str, container_args: str):
        attempt: int = 1
        max_attempts: int = 10
        old_container_args = container_args
        if self.create_container(cid_file, container_args=container_args):
            cid = self.get_cid_file()
            while not DockerCLIWrapper.docker_inspect(
                field="{{.State.Running}}", src_image=cid
            ):
                time.sleep(2)
                attempt += 1
                if attempt > max_attempts:
                    DockerCLIWrapper.run_docker_command("stop cid")
                    return True
            exit_code = DockerCLIWrapper.docker_inspect(
                field="{{.State.ExitCode}}", src_image=cid
            )
            if exit_code == 0:
                return True
            DockerCLIWrapper.run_docker_command(f"rm -v {cid}")
            self.cid_file.unlink()
        if old_container_args != "":
            self.container_args = old_container_args
        return False

    def npm_works(self):
        tempdir = mkdtemp(suffix="npm_test")
        self.cid_file = Path(tempdir) / "cid_npm_test"
        try:
            DockerCLIWrapper.run_docker_command(
                f'run --rm {self.image_name} /bin/bash -c "npm --version"'
            )
        except subprocess.CalledProcessError:
            logger.error(
                f"'npm --version' does not work inside the image {self.image_name}."
            )
            return False

        # TODO
        # Add {self.image_name}-testapp as soon as function `s2i_create_df` is ready.
        DockerCLIWrapper.run_docker_command(
            f"run -d {get_mount_ca_file()} --rm --cidfile={self.cid_file} {self.image_name}"
        )
        if not self.wait_for_cid():
            logger.error("Container did not create cidfile.")
            return False

        try:
            jquery_output = DockerCLIWrapper.run_docker_command(
                f"exec {self.get_cid_file(self.cid_file)} "
                f"/bin/bash -c "
                f"'npm --verbose install jquery && test -f node_modules/jquery/src/jquery.js'"
            )
        except subprocess.CalledProcessError:
            logger.error(
                f"npm could not install jquery inside the image ${self.image_name}."
            )
            return False
        if getenv("NPM_REGISTRY") and get_full_ca_file_path().exists():
            if get_os_environment("NPM_REGISTRY") in jquery_output:
                logger.error("Internal repository is NOT set. Even it is requested.")
                return False

        if self.cid_file.exists():
            DockerCLIWrapper.run_docker_command(
                f"stop {self.get_cid_file(self.cid_file)}"
            )
            self.cid_file.unlink()
        logger.info("Npm works.")
        return True

    def binary_found_from_df(self, binary: str = "", binary_path: str = "^/opt/rh"):
        tempdir = mkdtemp(suffix=f"{self.image_name}_binary")
        dockerfile = Path(tempdir) / "Dockerfile"
        logger.info(f"Testing {binary} in build from Dockerfile")
        content: str = f"""FROM {self.image_name}
RUN which {binary} | grep {binary_path}
        """
        with open(dockerfile, "w") as f:
            f.write(content)
        if not DockerCLIWrapper.run_docker_command(
            f"build -f {dockerfile} --no-cache {tempdir}", return_output=False
        ):
            logger.error(f"Failed to find {binary} in Dockerfile!")
            return False
        return True

    def check_latest_imagestreams(self):
        pass

    #     local latest_version=
    #     local test_lib_dir=
    #
    #     # Check only lines which starts with VERSIONS
    #     latest_version=$(grep '^VERSIONS' Makefile | rev | cut -d ' ' -f 1 | rev )
    #     test_lib_dir=$(dirname "$(readlink -f "$0")")
    #     python3 "${test_lib_dir}/check_imagestreams.py" "$latest_version"
    #

    def doc_content_old(self, strings: List) -> bool:
        logger.info("Testing documentation in the container image")
        files_to_check = ["help.1"]
        for f in files_to_check:
            doc_content = DockerCLIWrapper.docker_run_command(f'--rm {self.image_name} /bin/bash -c cat {f}')
            for term in strings:
                # test = re.search(f"{term}", doc_content)
                logger.info(f"ERROR: File /{f} does not contain '{term}'.")
                return False
            for term in ["TH", "PP", "SH"]:
                if term not in doc_content:
                    logger.info(f"ERROR: help.1 is probably not in troff or groff format, since {term} is missing")
                    return False
        return True

    #         # ct_doc_content_old [strings]
    #         # --------------------
    #         # Looks for occurence of stirngs in the documentation files and checks
    #         # the format of the files. Files examined: help.1
    #         # Argument: strings - strings expected to appear in the documentation
    #         # Uses: $IMAGE_NAME - name of the image being tested
    #         function
    #         ct_doc_content_old()
    #         {
    #             local
    #         tmpdir
    #         tmpdir =$(mktemp - d)
    #         local
    #         f
    #         : "  Testing documentation in the container image"
    #         # Extract the help files from the container
    #         # shellcheck disable=SC2043
    #         for f in help.1; do
    #         docker run --rm "${IMAGE_NAME}" / bin / bash -c "cat /${f}" >
    #         "${tmpdir}/$(basename "${f}")"
    #         # Check whether the files contain some important information
    #         for term in "$@"; do
    #         if ! grep -F -q -e "${term}" "${tmpdir}/$(basename "${f}")"; then
    #         echo "ERROR: File /${f} does not include '${term}'." > & 2
    #         return 1
    #         fi
    #
    #     done
    #     # Check whether the files use the correct format
    #     for term in TH PP SH; do
    #     if ! grep -q "^\.${term}" "${tmpdir}/help.1"; then
    #     echo
    #     "ERROR: /help.1 is probably not in troff or groff format, since
    #     '${term}' is missing." > & 2
    #     return 1
    #
    #
    # fi
    # done
    # done
    # : "  Success!"
    # }

    def test_response(self):
        pass

    def test_check_exec_env_vars(self, env_filter: str = "^X_SCLS=|/opt/rh|/opt/app-root"):
        check_envs = DockerCLIWrapper.docker_run_command(f'--rm {self.image_name} /bin/bash -c env')
        logger.debug(f"Run envs {check_envs}")
        self.create_container(cid_file="exec_env_vars", container_args="bash -c 'sleep 1000'")
        loop_envs = DockerCLIWrapper.run_docker_command(f"exec {self.get_cid_file(self.cid_file)} env")
        self.test_check_envs_set(env_filter=env_filter, check_envs=check_envs, loop_envs=loop_envs)

    def test_check_envs_set(self, env_filter: str, check_envs: str, loop_envs: str, env_format="VALUE"):
        fields_to_check: List = [
            x for x in loop_envs.split('\n') if re.findall(env_filter, x) and not x.startswith("PWD=")
        ]
        for field in fields_to_check:
            var_name, stripped = field.split('=', 2)
            filtered_envs = [x for x in check_envs.split('\n') if x.startswith(f"{var_name}=")]
            if not filtered_envs:
                logger.error(f"{var_name} not found during 'docker exec'")
                return False
            filter_envs = ''.join(filtered_envs)
            for value in stripped.split(':'):
                # If the value checked does not go through env_filter we do not care about it
                ret = re.findall(env_filter, value)
                if not ret:
                    continue
                new_env = env_format.replace('VALUE', value)
                find_env = re.findall(rf"{new_env}", filter_envs)
                if not find_env:
                    logger.error(f"Value {value} is missing from variable {var_name}")
                    logger.error(filtered_envs)
                    return False
        return True

    def test_helm_chart(self, path: str,  package_name: str, version: str, test_check_string: str):
        hc = HelmChartsAPI(path=path, package_name=package_name, version=version)
        if not hc.helm_package():
            return False
        hc.helm_installation()
        hc.test_helm_chart(test_check_string)

    def test_helm_chart_imagestreams(self, path: str, package_name: str, version: str, registry: str) -> bool:
        hc = HelmChartsAPI(path=path, package_name=package_name, version=version)
        if not hc.helm_package():
            return False
        hc.helm_installation()
        hc.check_imagestreams(version, registry)
        return True
